require 'spec_helper'

class MockAggregator < Almicube::Aggregator::Base
end

describe Almicube::Ranking::DataRanking do
  describe "#selector" do
    let(:ranking) { described_class.new selector: selector }

    describe "sets the mock selector" do
      let(:selector) { MockSelector.new records: records }
      let(:records) { [double(), double()] }
      it { expect(ranking.records).to eq records }
      it { expect(ranking.selector).to eq selector }
    end

    describe "does not set the selector" do
      let(:selector) { nil }
      it { expect(ranking.selector).to be_kind_of Almicube::Selector::AllSelector }
    end
  end

  describe "#aggregator" do
    let(:ranking) { described_class.new aggregator: aggregator }

    describe "set the mock aggregator" do
      let(:aggregator) { MockAggregator.new }
      before { aggregator.should_receive(:aggregate).with(ranking) }
      it { ranking.aggregate }
    end
  end

  describe "#per_page" do
    let(:ranking) { described_class.new per_page: 10 }
    it { expect(ranking.per_page).to eq 10 }
  end

  describe "#default_score" do
    context "don't set default_score" do
      let(:ranking) { described_class.new }
      it { expect(ranking.default_score).to eq 0 }
    end

    context "sets default_score by options" do
      let(:ranking) { described_class.new default_score: 0 }
      it { expect(ranking.default_score).to eq 0 }
    end
  end

  describe "#attribute_name" do
    context "don't set attribute_name" do
      let(:ranking) { described_class.new }
      it { expect(ranking.attribute_name).to eq :score }
    end

    context "sets attribute_name by options" do
      %i[score access].each do |name|
        describe "to #{name}" do
          let(:ranking) { described_class.new attribute_name: :score }
          it { expect(ranking.attribute_name).to eq :score }
        end
      end
    end
  end

  describe "#date" do
    context "don't set date" do
      let(:ranking) { described_class.new }
      it { expect(ranking.date).to eq Date.today }
    end

    context "sets date by options" do
      let(:date) { Date.today - 12.days }
      let(:ranking) { described_class.new date: date }
      it { expect(ranking.date).to eq date }
    end
  end

  describe "#class_name" do
    let(:ranking) { described_class.new class_name: MockModel }
    it { expect(ranking.class_name).to eq MockModel }
  end

  describe "#data?" do
    let(:ranking) { described_class.new }
    it { expect(ranking).to be_data }
  end

  describe "#key" do
    context "with prefix and suffix" do
      let(:ranking) { described_class.new prefix: "aaa", suffix: "bbb" }
      it { expect(ranking.key).to match /^aaa:/ }
      it { expect(ranking.key).to match /:bbb$/ }
    end

    describe "differents between two days data" do
      let(:today) { Date.today }
      let(:ranking1) { described_class.new date: today }
      let(:ranking2) { described_class.new date: today - 1.days }

      it { expect(ranking1.key).to match /#{today.strftime("%Y%m%d")}/ }
      it { expect(ranking1.key).to_not match /#{ranking2.key}/ }
    end

    describe "include class name" do
      let(:ranking) { described_class.new class_name: MockModel }

      it { expect(ranking.key).to match /#{MockModel.to_s}/ }
    end

    describe "include attribute name" do
      let(:ranking) { described_class.new attribute_name: :score }

      it { expect(ranking.key).to match /score/ }
    end

    describe "include data flag" do
      context "without specific selector" do
        let(:ranking) { described_class.new }

        it { expect(ranking.key).to match /:data:/ }
      end
    end

    describe "exclude bundler name" do
      let(:ranking) { described_class.new bundler: MockBundler.new }

      it { expect(ranking.key).not_to match /bundle/ }
    end
  end

  describe "#options" do
    let(:ranking) { described_class.new }
    let(:options) { ranking.options }

    %i[selector aggregator date per_page prefix suffix].each do |key|
      it { expect(options).to have_key key }
    end

    %i[type].each { |key| it { expect(options).not_to have_key key } }
  end

  describe "#in" do
    context "with correct instance" do
      let(:ranking) { described_class.new class_name: MockModel }
      let(:target) { double( mock_models: [] ) }

      before do
        Almicube::Selector::HasManySelector.any_instance.stub(:"ranking=")
      end

      it { expect(ranking.in(target)).to be_kind_of Almicube::Ranking::SubRanking }
    end

    context "without correct instance" do
      let(:ranking) { described_class.new }
      let(:target) { double() }

      before do
        Almicube::Selector::HasManySelector.any_instance.stub(:"ranking=")
      end

      it { expect(ranking.in(target)).not_to be_kind_of Almicube::Ranking::SubRanking }
    end
  end

  describe "#exists?" do
    let(:ranking) { described_class.new }
    let(:key) { "sample" }
    before do
      @connection = Redis.new
      Almicube::Config.config.stub(:connection) { @connection }
      ranking.stub(:key).and_return(key)
    end

    context "exists key in redis" do
      before { @connection.del key }
      it { expect(ranking).not_to be_exists }
    end

    context "don't exist key in redis" do
      before { @connection.zadd key, 10, 10 }
      it { expect(ranking).to be_exists }
    end
  end

  describe "#ranged" do
    let(:ranking) { described_class.new }
    it { expect(ranking.ranged(:yesterday)).to be_kind_of Almicube::Ranking::SubRanking }
  end
end
